#!/usr/bin/Rscript --vanilla

## Profile generator for SNORT AnomalyDetection v2.5

##quit status:
#   0 - OK
#   1 - help
#   2 - error  

##dependencies
#parsing script params
require('getopt', quietly=T);  
#tseries object, HW model
require('quadprog', quietly=T);
require('zoo', quietly=T);
require('tseries', quietly=T);

#getopt specification
#   columns:
#       1   long flag name
#       2   short flag name
#       3   argument mask:  0 - no argument; 1 - requied argument; 2 - optional argument
#       4   argument data type: logical, integer, double, complex, character
#       5   (optional) brief description
spec <- matrix(c(
    'help',     'h',    0,  "logical",  'Show this help',
    'verbose',  'v',    0,  "logical",  'Verbose mode',
    'log',      'l',    1,  "character", 'Log file path',
    'profile',  'p',    1,  "character", 'Create profile file (path)',
    'evaluator','e',    1,  "character", 'Create profile file for AD_Evaluator (path)',
    'pattern',  'P',    1,  "character", 'Create pattern file (path)',
    'save',     's',    1,  "character", 'Save model parameters (path)',
    'method',   'm',    1,  "character", 'Method of profile generation (see below)',
    'ahead',    'a',    1,  "character",  'Number of future periods to predict',
    'scale',	'd',    1,  "double",     'Scaling factor for the width of the confidence band (default=1)',
    'avg',      NA,     1,  "character", 'AVG model parameters',
    'naive',    NA,     1,  "character", 'NAIVE model parameters',
    'ar',       NA,     1,  "character", 'AR model parameters',
    'hw',       NA,     1,  "character", 'HW model parameters',
    'brutlag',  NA,     1,  "character", 'BRUTLAG model parameters'
), ncol=5, byrow=TRUE);

## Shows help message
# usage automatically generated from getopt spec
#wycinanie '-NA' z pojedynczych parametrów
printHelp <- function(){
    write(c("Profile Generator for SNORT AnomalyDetection\n",
	paste(unlist(strsplit(getopt(spec, command="ad_profilegenerator.r", usage=TRUE),"-NA")),collapse="  "),
"Methods of profile generation:
 AVG\t\tMoving average (default)
 NAIVE\t\tNaive method
 AR\t\tAutoregressive time series model
 HW\t\tHolt-Winters model
 BRUTLAG\tHW model with Brutlag\'s confidence band
 
Number of future periods to predict:
 WEEK\t\tCreate profile for one week (default)
 MONTH\t\tCreate profile for four weeks
 <integer>\tSet number of future periods 
 
AVG model parameters:
    format: \"<type>,<n>\"
  where <type> in:
 LAST\t\tCompute moving average from <n> last values
 DAILY\t\tCompute moving average for <n> lat hour-of-day
 WEEKLY\t\tCompute moving average for <n> last day-of-weeks (default, n=3)
 
NAIVE model parameters:
    format: \"<type>\"
  where <type> in:
 LAST\t\tCompute naive model for <n> last values
 DAILY\t\tCompute naive model for <n> lat hour-of-day
 WEEKLY\t\tCompute naive model for <n> last day-of-weeks (default, n=3)
 
AR model parameters:
    format: \"<periodicity>,<fit_method>\"
  where <periodicity> in:
 DAILY\t\tCompute model with daily periodicity
 WEEKLY\t\tCompute model with weekly periodicity (default)
        <fit_method> in:
 YW\t\tYule-Walker\'s method (default)
 BURG\t\tBurg\'s method
 MLE\t\tMaximum Likelihood Estimation 
 OLS\t\tOrdinary Least Squares
 
HW model parameters:
 DAILY\t\tCompute model with daily periodicity
 WEEKLY\t\tCompute model with weekly periodicity (default)
 
BRUTLAG model parameters:
 DAILY\t\tCompute model with daily periodicity
 WEEKLY\t\tCompute model with weekly periodicity (default)
 **Caution** for Brutlag model you should take care of \'--scale\' parameter.
 Sensible values of \'--scale\' are between 2 and 3.
"),stdout())
}

## Shows messages & errors
#   message         - text to show
#   check.verbose   - check if program is in verbose mode
#   error           - TRUE if message is error <- always showed
showMessage <- function(message, check.verbose=TRUE, error=FALSE){
	if(!is.character(message)) stop("message must be a 'character' type")

	message <- c("ProfileGen:", message)
	
	if(error){
	    message <- append(message, "ERROR:", 1)
	    write(paste(message, collapse=" "), stderr())
	}else if(check.verbose){
		if(opt$verbose) write(paste(message, collapse=" "), stdout())
	} else {
		write(paste(message, collapse=" "), stdout())
	}
}

##converts hour HH:MM:SS to sample number
hourToSampleNumber <- function(hour, interval){
	if(length(hour) != 1) stop("incorrect input hour format.")
	if(!is.numeric(interval)) stop("incorrect input interval format.")

	data <- as.numeric(unlist(strsplit(as.character(hour),":")))
	seconds <- data[1]*3600+data[2]*60+data[3]

	return((floor(seconds/interval)+1))
}

#parsing parameters
opt <- getopt(spec)

#if asked for help:
if ( !is.null(opt$help) ) {
   printHelp()
   q(status=1);
}

##TODO: przy wstawianiu nowej metody obliczeń dodać tutaj
#zabezpieczenie przed ustawieniem parametrów modelu bez wyboru modelu (nie dotyczy AVG)
if ( is.null(opt$method) ){
    if ( is.character(opt$ar)||is.character(opt$hw)||is.character(opt$brutlag)||is.character(opt$naive) ){
        showMessage("Choose model BEFORE specifying model parameters",error=T)
        q(status=2)
    }
}

#set some defaults for the options that were not specified
if ( is.null(opt$verbose    ))  { opt$verbose   <- FALSE                        }
if ( is.null(opt$log        ))  { opt$log       <- "czysty.csv"					}
if ( is.null(opt$scale      ))  { opt$scale 	<- 1                            }
if ( is.null(opt$ahead      ))  { opt$ahead     <- "WEEK"                       }
if ( is.null(opt$method     ))  { opt$method    <- "AVG"                      	}

opt$method <- toupper(opt$method)
opt$ahead <- toupper(opt$ahead)

showMessage("Starting Profile Generator for SNORT AnomalyDetection")

#jeśli nie wybrane zapisywanie profilu, wzorca ani parametrów modelu
if(is.null(opt$profile)&&is.null(opt$pattern)&&is.null(opt$save)&&is.null(opt$evaluator)){
    printHelp()
    showMessage("Nothing to do. Bye")
    quit(status=0)
}

showMessage(c("Using log file:", opt$log))
if (is.character(opt$profile)) { showMessage(c("Saving profile to:", opt$profile)) }
if (is.character(opt$pattern)) { showMessage(c("Saving pattern to:", opt$pattern)) }
if (is.character(opt$save)) { showMessage(c("Saving model parameters to:", opt$save)) }
showMessage(c("Using method of prediction:", opt$method))

##reading log file
log.data <- read.csv(opt$log)

##setting static column positions
log.column.date <- 1
log.column.hour <- 2
log.column.day <- 3
log.column.interval <- 4

log.interval <- log.data[1,log.column.interval]

showMessage(c("Log interval:",log.interval))

if ( opt$ahead == "WEEK" ){ 
    opt$ahead <- 604800/log.interval
    opt$aheadWeek <- TRUE
} else if ( opt$ahead == "MONTH" ){ 
    opt$ahead <- (604800/log.interval) * 4
} else {
    opt$ahead <- as.integer(opt$ahead)
    
    if ( is.na(opt$ahead) ) {
        showMessage("Incorrect value of '-a' option. Exiting", error=T)
        q(status=2)
    }
    
    if ( opt$ahead <= 0 ) {
        showMessage("Value of '-a' option MUST be positive. Exiting", error=T)
        q(status=2)
    }
}

if ( is.character(opt$profile)||is.character(opt$evaluator) ){
    showMessage(c("Number of values to predict:", opt$ahead))
}

if ( is.numeric(opt$scale) ){
    showMessage(c("Deviation scale:", opt$scale))
}

##checking if all intervals are the same
if((length(which(log.data[,log.column.interval] == log.interval)) == nrow(log.data)) == FALSE){ 
	showMessage("Time interval error. Exiting", error=T)
	quit(status=2)
}

#zapisanie pierwszej daty z loga
log.first.date <- as.Date(log.data[1,log.column.date],"%d-%m-%y")

#numer pierwszej próbki
log.first.sample.no <- hourToSampleNumber(log.data[1,log.column.hour],log.interval)

#zapisanie czasu referencyjnego (ostatni czas z loga)
temp <- paste(log.data[nrow(log.data),log.column.date],log.data[nrow(log.data),log.column.hour])
referencetimestamp <- strptime(temp,"%d-%m-%y %H:%M:%S",tz="EST5EDT")	

## matrixes for profile.data data
profile.data <- matrix(0, nrow=opt$ahead, ncol=((ncol(log.data)*2)-4))
evaluator.data <- matrix(0, nrow=opt$ahead, ncol=(ncol(log.data)))

pattern.dates<- log.data[,1:3]   #dates from logfile
## removing date, hour, "Day of the Week" & "Time interval" columns from log
log.data <- subset(log.data, select=-c(1:4))

if(opt$method == "AR"){
#-------------------------------------------------------------------------------
#               Autoreggresive model
#-------------------------------------------------------------------------------
    ##periodicity
    if ( is.null(opt$ar))  { opt$ar <- "WEEKLY,YW"}
    
    opt$ar <- toupper(opt$ar)
    
    opt$ar <- unlist(strsplit(opt$ar,","))
    
    opt$ar <- toupper(opt$ar)
    
    if(opt$ar[1] == "DAILY"){
        profile.config.frequency <- 86400 / log.interval	##daily periodicity
    } else if (opt$ar[1] == "WEEKLY") {
        profile.config.frequency <- 604800 / log.interval	##weekly periodicity
    } else {
        showMessage("Incorrect AR model periodicity. Exiting", error=T)
	    quit(status=2)
    }
    
    if((opt$ar[2] == "YW")||(opt$ar[2] == "BURG")||(opt$ar[2] == "OLS")||(opt$ar[2] == "MLE")){
        opt$ar[2] <- tolower(opt$ar[2])
    } else {
        showMessage("Incorrect AR fitting method. Exiting", error=T)
	    quit(status=2)
    }
    
    #macierz na parametry modelu
    #zapisywane tylko dla PATTERN
    #zawiera: ar, method,x.mean,resid1...residp
    model.parameters.data.nrow <- profile.config.frequency+52+50
    model.parameters.names <- c(paste(opt$method,opt$ar[1],toupper(opt$ar[2])),names(log.data))
    model.parameters.data <- matrix(NA, nrow=model.parameters.data.nrow,ncol=(ncol(log.data)+1))
    
    for (x in c(1:50)){
        model.parameters.data[x,1] <- paste("ar_",x,sep='')    
    }
    model.parameters.data[51,1] <- "method"
    model.parameters.data[52,1] <- "x.mean"
    for (x in c(1:profile.config.frequency)){
        model.parameters.data[(x+52),1] <- paste("resid_",x,sep='')    
    }
    for (x in c(0:49)){
        model.parameters.data[(x+53+profile.config.frequency),1] <- paste("aic_",x,sep='')    
    }
    
    if (is.character(opt$pattern)){
        #plik PATTERN
        pattern.data <- matrix(0, nrow=(nrow(log.data)-profile.config.frequency), ncol=(ncol(log.data)+4)) 
        #długosc loga - n*okres, bo wartości modelu dostępne są począwszy od (n*okres)+1 
        
        for(column.log in c(1:ncol(log.data))){
        	## creating time-series object for current column
		    ts_obj <- ts(log.data[1:profile.config.frequency,column.log], frequency=profile.config.frequency, start=c(as.numeric(log.first.date),log.first.sample.no))

		    ## Fit an autoregressive time series model to the data
		    model <- ar(ts_obj,method=opt$ar[2])
		    
		    #zapisanie parametrów modelu
		    model.parameters.data[1:50,(column.log+1)]<- as.character(model$ar[1:50]) 
		    model.parameters.data[51,(column.log+1)]<- as.character(model$method) 
		    model.parameters.data[52,(column.log+1)]<- as.character(model$x.mean) 
		    model.parameters.data[53:(profile.config.frequency+52),(column.log+1)] <- as.character(model$resid) 
		    model.parameters.data[(profile.config.frequency+53):nrow(model.parameters.data),(column.log+1)] <- as.character(model$aic[1:50])
		    
		    ## Prediction
		    pred <- predict(model, n.ahead=(nrow(log.data)-profile.config.frequency))
		    values <- as.numeric(pred$pred)
		
		    pattern.data[,column.log+4] <- as.character(round(values))
	    }
        
        #date
        pattern.data[,1] <- as.character(pattern.dates[(profile.config.frequency+1):nrow(pattern.dates),1])
        #weekday (converting from date to weekday 1...7)
        pattern.data[,2] <- format(strptime(pattern.dates[(profile.config.frequency+1):nrow(pattern.dates),1],"%d-%m-%y",tz="EST5EDT"),"%u")
        #hour
        pattern.data[,3] <- as.character(pattern.dates[(profile.config.frequency+1):nrow(pattern.dates),2])
        #interval
        pattern.data[,4] <- log.interval
    }
    
    
    if ((is.character(opt$profile))||(is.character(opt$evaluator))){
	    for(column.log in c(1:ncol(log.data))){
        	## creating time-series object for current column
		    ts_obj <- ts(log.data[,column.log], frequency=profile.config.frequency, start=c(as.numeric(log.first.date),log.first.sample.no))

		    ## Fit an autoregressive time series model to the data
		    model <- ar(ts_obj,method=opt$ar[2])
		    ## Prediction
		    pred <- predict(model, n.ahead=opt$ahead)
		    values <- as.numeric(pred)
		
		    evaluator.data[,column.log+4] <- as.character(round(values))
		
			#standard deviation of population
			std_dev_data <- log.data[,column.log]
			std_dev_err <- std_dev_data - mean(std_dev_data)
		
			std_dev_pop <- sqrt(sum(std_dev_err^2)/length(std_dev_data))
        	deviation <- (opt$scale * std_dev_pop)
            
            column.pred <- (2 * column.log) + 3
            profile.data[,column.pred] <- as.character(round(values - deviation))
            profile.data[,column.pred+1] <- as.character(round(values + deviation))
	    }
	}
} else if((opt$method == "HW")||(opt$method == "BRUTLAG")){
#-------------------------------------------------------------------------------
#               Holt-Winters & Brutlag model
#-------------------------------------------------------------------------------
    if(opt$method == "HW"){
		##periodicity
		if ( is.null(opt$hw))  { opt$hw <- "WEEKLY"}
		
		opt$hw <- toupper(opt$hw)
		
		if(opt$hw == "DAILY"){
		    profile.config.frequency <- 86400 / log.interval	##daily periodicity
		} else if (opt$hw == "WEEKLY") {
		    profile.config.frequency <- 604800 / log.interval	##weekly periodicity
		} else {
		    showMessage("Incorrect HW model parameters. Exiting", error=T)
			quit(status=2)
		}

		#macierz na parametry modelu
		#zawiera: alpha, beta, gamma, a, b, s1...sp, SSE
		model.parameters.data.nrow <- profile.config.frequency+6
		model.parameters.names <- c(paste(opt$method,opt$hw),names(log.data))
		model.parameters.data <- matrix(0, nrow=model.parameters.data.nrow,ncol=(ncol(log.data)+1))
    } else {
    	##periodicity
		if ( is.null(opt$brutlag))  { opt$brutlag <- "WEEKLY"}
		
		opt$brutlag <- toupper(opt$brutlag)
		
		if(opt$brutlag == "DAILY"){
		    profile.config.frequency <- 86400 / log.interval	##daily periodicity
		} else if (opt$brutlag == "WEEKLY") {
		    profile.config.frequency <- 604800 / log.interval	##weekly periodicity
		} else {
		    showMessage("Incorrect BRUTLAG model parameters. Exiting", error=T)
			quit(status=2)
		}

		#macierz na parametry modelu
		#zawiera: alpha, beta, gamma, a, b, s1...sp, SSE
		model.parameters.data.nrow <- profile.config.frequency+6
		model.parameters.names <- c(paste(opt$method,opt$brutlag),names(log.data))
		model.parameters.data <- matrix(0, nrow=model.parameters.data.nrow,ncol=(ncol(log.data)+1))
    }
    
    model.parameters.data[1,1] <- "alpha"
    model.parameters.data[2,1] <- "beta"
    model.parameters.data[3,1] <- "gamma"
    model.parameters.data[4,1] <- "a"
    model.parameters.data[5,1] <- "b"
    for (x in c(1:profile.config.frequency)){
        model.parameters.data[(x+5),1] <- paste("s",x,sep='')    
    }
    model.parameters.data[model.parameters.data.nrow,1] <- "SSE"
    
    #macierz na wartości modelu
    pattern.data <- matrix(0, nrow=(nrow(log.data)-profile.config.frequency), ncol=(ncol(log.data)+4)) 
    #długosc loga - pierwszy okres, bo wartości modelu dostępne są począwszy od drugiego okresu

	for(column.log in c(1:ncol(log.data))){
		## creating time-series object for current column
		ts_obj <- ts(log.data[,column.log], frequency=profile.config.frequency, start=c(as.numeric(log.first.date),log.first.sample.no))

		## Computes Holt-Winters Filtering of a given time series.
		## Parameters are determined by minimizing the squared prediction error.
		model <- HoltWinters(ts_obj)
		
		#zapisanie parametrów modelu
		model.parameters.data[1,(column.log+1)]<- as.character(model$alpha) #alpha
		model.parameters.data[2,(column.log+1)]<- as.character(model$beta)  #beta
		model.parameters.data[3,(column.log+1)]<- as.character(model$gamma) #gamma
		model.parameters.data[4:(profile.config.frequency+5),(column.log+1)] <- as.character(model$coefficients)   #a, b, s1 ... sp
		model.parameters.data[model.parameters.data.nrow,(column.log+1)] <- as.character(model$SSE)
		
		pattern.data[,column.log+4] <- as.character(round(model$fitted[,1]))
		
		## Prediction
		pred <- predict(model, n.ahead=opt$ahead)
		values <- as.numeric(pred)
		
		evaluator.data[,column.log+4] <- as.character(round(values))
		
		#width of confidence band
		if(opt$method=="HW"){
			#standard deviation of population
			std_dev_data <- log.data[,column.log]
			std_dev_err <- std_dev_data - mean(std_dev_data)
		
			std_dev_pop <- sqrt(sum(std_dev_err^2)/length(std_dev_data))
		    deviation <- (opt$scale * std_dev_pop)
        } else {
            #wartości wyliczane
        	val.pred<-as.numeric(model$fitted[(length(model$fitted[,1])-profile.config.frequency+1):length(model$fitted[,1]),1])
        	#wartości rzeczywiste
        	val.real<-model$x[(length(model$x)-profile.config.frequency+1):length(model$x)]
            
            #pierwsza część wzoru Brutlaga na szerokość wstążki
            val.brutlag <- model$gamma*abs(val.real-val.pred)
            val.brutlag2 <- val.brutlag/model$gamma
            val.brutlag <- val.brutlag + val.brutlag2
            
            val.brutlag<- rep(val.brutlag,length.out=opt$ahead)
            
        	deviation <- (opt$scale * val.brutlag)
        }
        column.pred <- (2 * column.log) + 3
        profile.data[,column.pred] <- as.character(round(values - deviation))
        profile.data[,column.pred+1] <- as.character(round(values + deviation))
	}
	
	#date
	pattern.data[,1] <- as.character(pattern.dates[(profile.config.frequency+1):nrow(pattern.dates),1])
	#weekday (converting from date to weekday 1...7)
	pattern.data[,2] <- format(strptime(pattern.dates[(profile.config.frequency+1):nrow(pattern.dates),1],"%d-%m-%y",tz="EST5EDT"),"%u")
	#hour
	pattern.data[,3] <- as.character(pattern.dates[(profile.config.frequency+1):nrow(pattern.dates),2])
	#interval
	pattern.data[,4] <- log.interval
	
} else if((opt$method == "AVG")||(opt$method == "NAIVE")){
#-------------------------------------------------------------------------------
#               Moving average & naive method
#-------------------------------------------------------------------------------

    
    if(opt$method == "AVG"){
		##periodicity
		if ( is.null(opt$avg))  { opt$avg <- "WEEKLY,3"}
    
	    opt$avg <- toupper(opt$avg)
    
		opt$avg <- unlist(strsplit(opt$avg,","))
		
		no.of.periods <- as.integer(opt$avg[2])
		
		if ( is.na(no.of.periods) ) {
		    showMessage("Incorrect value of '--avg' option. Exiting", error=T)
		    q(status=2)
		}
		
		if ( no.of.periods <= 0 ) {
		    showMessage("Second value of '--avg' option MUST be positive. Exiting", error=T)
		    q(status=2)
		}
		
		model.parameters.data <- matrix(0, nrow=3,ncol=2)
		model.parameters.names <- FALSE
		model.parameters.data[1,1] <- "model"
		model.parameters.data[1,2] <- "AVG"
		model.parameters.data[2,1] <- "periodicity"
		model.parameters.data[2,2] <- opt$avg[1]
		model.parameters.data[3,1] <- "n"
		model.parameters.data[3,2] <- opt$avg[2]
		
		if(opt$avg[1] == "LAST"){
		    #LAST
		    profile.config.frequency <- 1
		    messagePeriods<-"rows."
		} else if(opt$avg[1] == "DAILY"){
		    #DAILY periodicity
		    profile.config.frequency <- 86400 / log.interval	##daily periodicity
		    messagePeriods<-"full days."
		} else if(opt$avg[1] == "WEEKLY"){
		    #WEEKLY periodicity
		    profile.config.frequency <- 604800 / log.interval	##weekly periodicity
		    messagePeriods<-"full weeks."
		} else {
		    showMessage("Incorrect AVG model periodicity. Exiting", error=T)
			    quit(status=2)
		}
		
		#check if log isn't too short
		if( nrow(log.data) < no.of.periods*profile.config.frequency ){
		    showMessage(paste("Log must contain at least",no.of.periods,messagePeriods,"Exiting"), error=T)
		    q(status=2)
		}
    } else {
   	    if ( is.null(opt$naive))  { opt$naive <- "WEEKLY"}
    
    	opt$naive <- toupper(opt$naive)
    
		no.of.periods <- 1
		
		model.parameters.data <- matrix(0, nrow=2,ncol=2)
		model.parameters.names <- FALSE
		model.parameters.data[1,1] <- "model"
		model.parameters.data[1,2] <- "NAIVE"
		model.parameters.data[2,1] <- "periodicity"
		model.parameters.data[2,2] <- opt$naive
		
		if(opt$naive == "LAST"){
		    #LAST
		    profile.config.frequency <- 1
		    messagePeriods<-"row."
		} else if(opt$naive == "DAILY"){
		    #DAILY periodicity
		    profile.config.frequency <- 86400 / log.interval	##daily periodicity
		    messagePeriods<-"full day."
		} else if(opt$naive == "WEEKLY"){
		    #WEEKLY periodicity
		    profile.config.frequency <- 604800 / log.interval	##weekly periodicity
		    messagePeriods<-"full week."
		} else {
		    showMessage("Incorrect NAIVE model periodicity. Exiting", error=T)
			    quit(status=2)
		}
		
		#check if log isn't too short
		if( nrow(log.data) < profile.config.frequency ){
		    showMessage(paste("Log must contain at least one",messagePeriods,"Exiting"), error=T)
		    q(status=2)
		}
    }
    
    if (is.character(opt$pattern)){
        #plik PATTERN
        pattern.data <- matrix(0, nrow=(nrow(log.data)-no.of.periods*profile.config.frequency), ncol=(ncol(log.data)+4)) 
        #długosc loga - n*okres, bo wartości modelu dostępne są począwszy od (n*okres)+1 
        
        log.rows.pattern <- seq(0, by=profile.config.frequency, length.out=no.of.periods)
        
        for(column.log in c(1:ncol(log.data))){
            #liczenie średniej z n wartości dla danej kolumny
            for(model.row in c(1:nrow(pattern.data))){                    
                pattern.data[model.row,column.log+4] <- as.character(round(mean(as.numeric(log.data[(log.rows.pattern+model.row),column.log]))))
            }
        }
        
        #date
        pattern.data[,1] <- as.character(pattern.dates[((no.of.periods*profile.config.frequency)+1):nrow(pattern.dates),1])
        #weekday (converting from date to weekday 1...7)
        pattern.data[,2] <- format(strptime(pattern.dates[((no.of.periods*profile.config.frequency)+1):nrow(pattern.dates),1],"%d-%m-%y",tz="EST5EDT"),"%u")
        #hour
        pattern.data[,3] <- as.character(pattern.dates[((no.of.periods*profile.config.frequency)+1):nrow(pattern.dates),2])
        #interval
        pattern.data[,4] <- log.interval
    }
    
    if (is.character(opt$profile)||is.character(opt$evaluator)){
    #Tworzenie pliku dla evaluatora
        #obcięcie pliku loga
        log.data<-as.matrix(log.data[((nrow(log.data)-(no.of.periods*profile.config.frequency)+1)):nrow(log.data),])
        
        #sekwencja
        log.rows.pattern <- seq(0, by=profile.config.frequency, length.out=no.of.periods)
        
        for(column.log in c(1:ncol(log.data))){
            #liczenie średniej z n wartości dla danej kolumny
            for(model.row in c(1:opt$ahead)){
                    evaluator.data[model.row,column.log+4] <- as.character(round(mean(as.numeric(c(log.data[,column.log],evaluator.data[,column.log+4])[log.rows.pattern+model.row]))))
            }
        }
    }
    
    if (is.character(opt$profile)){
        #profil liczony na podstawie wartości pliku evaluatora:
        for(column.log in c(1:ncol(log.data))){
            column.eva <- column.log+4
            column.pro <- (column.log*2)+3
            
            temp <- as.integer(evaluator.data[,column.eva])
            
            #standard deviation of population
			std_dev_data <- log.data[,column.log]
			std_dev_err <- std_dev_data - mean(std_dev_data)
			
			std_dev_pop <- sqrt(sum(std_dev_err^2)/length(std_dev_err))
            deviation <- (opt$scale * std_dev_pop)
            
            
#			showMessage(c(column.log,"dlugosc:", length(std_dev_err^2)))		            		
#			showMessage(c(column.log,"sumy^2:", sum(std_dev_err^2)))		
#            showMessage(c(column.log,"dev.pop * opt:", (opt$scale*std_dev_pop)))		
#			showMessage(c(column.log,"dev.pop:", std_dev_pop))		
#			showMessage(c(column.log,"deviation:", deviation[1]))
#            			showMessage(c(column.log,"temp:", temp[1]))
            
            profile.data[,column.pro]<- as.character(round(temp-deviation))
            profile.data[,(column.pro+1)]<- as.character(round(temp+deviation))
        }
    }
} else {
	showMessage("Invalid model. Exiting", error=T)
	write(getopt(spec, command="ad_profilegenerator.r", usage=TRUE),stdout())
	showMessage("Use '-h' option to show full help.", FALSE)
	quit(status=2)
}

#removing negative values
evaluator.data[which(evaluator.data < 0)] <- 0
profile.data[which(profile.data < 0)] <- 0

##data field
referencetimestamp <- referencetimestamp + log.interval
for(i in c(1:nrow(profile.data))){
    profile.data[i,1] <- format(referencetimestamp,"%d-%m-%y")
	profile.data[i,2] <- format(referencetimestamp,"%u")
	profile.data[i,3] <- format(referencetimestamp,"%H:%M:%S")
	referencetimestamp <- referencetimestamp+log.interval
}

if (!is.null(opt$aheadWeek)) {
    if( opt$aheadWeek == TRUE ){
        ##profile for exacly one week:
        # data column = 0
        profile.data[,1] <- 0
    }
}

##log interval field
profile.data[,4] <- log.interval
evaluator.data[,1:4] <- profile.data[,1:4]

##saving files
#   profile
if (is.character(opt$profile)) { 
write.table(profile.data, file=opt$profile, sep=",", col.names=FALSE, row.names=FALSE, append=FALSE, quote=FALSE)
}
#   evaluator file
if (is.character(opt$evaluator)) { 
write.table(evaluator.data, file=opt$evaluator, sep=",", col.names=FALSE, row.names=FALSE, append=FALSE, quote=FALSE)
}
#   pattern file
if (is.character(opt$pattern)) { 
write.table(pattern.data, file=opt$pattern, sep=",", col.names=FALSE, row.names=FALSE, append=FALSE, quote=FALSE)
}
#   model parameters file
if (is.character(opt$save)) {
write.table(model.parameters.data, file=opt$save, sep=",", col.names=model.parameters.names, row.names=FALSE, append=FALSE, quote=FALSE)
}

showMessage(c("Total time elapsed:", as.character(proc.time()[3]),"sec"))


